@@ -1,17 +1,18 @@
 from .action_client import ActionClient
+from .cli.auth import get_authorizer_for_scope
 from .client_helpers import create_action_client, create_flows_client
-from .flows_client import FlowsClient
+from .flows_client import FlowsClient, validate_flow_definition
 from .graphviz_rendering import graphviz_format, state_colors_for_log
 from .queues_client import QueuesClient, create_queues_client
-from .token_management import get_authorizer_for_scope
 
-__version__ = "0.10.7"
+__version__ = "0.11.0"
 
 __all__ = (
     "ActionClient",
     "create_action_client",
     "FlowsClient",
     "create_flows_client",
+    "validate_flow_definition",
     "QueuesClient",
     "create_queues_client",
     "get_authorizer_for_scope",
