@@ -1,9 +1,7 @@
-import json
-from enum import Enum
+import functools
 from typing import List
 
 import typer
-import yaml
 from globus_sdk import GlobusAPIError
 
 from globus_automate_client.cli.callbacks import (
@@ -11,28 +9,19 @@ from globus_automate_client.cli.callbacks import (
     principal_validator,
     url_validator_callback,
 )
-from globus_automate_client.cli.constants import InputFormat
+from globus_automate_client.cli.constants import InputFormat, OutputFormat
 from globus_automate_client.cli.helpers import (
     format_and_echo,
     process_input,
+    request_runner,
     verbosity_option,
 )
+from globus_automate_client.cli.rich_rendering import live_content
 from globus_automate_client.client_helpers import create_action_client
 
 app = typer.Typer(short_help="Manage Globus Automate Actions")
 
 
-class ActionOutputFormat(str, Enum):
-    json = "json"
-    yaml = "yaml"
-
-    def get_dumper(self):
-        if self is self.json:
-            return json.dumps
-        elif self is self.yaml:
-            return yaml.dump
-
-
 @app.command("introspect")
 def action_introspect(
     action_url: str = typer.Option(
@@ -47,8 +36,8 @@ def action_introspect(
         callback=url_validator_callback,
     ),
     verbose: bool = verbosity_option,
-    output_format: ActionOutputFormat = typer.Option(
-        ActionOutputFormat.json,
+    output_format: OutputFormat = typer.Option(
+        OutputFormat.json,
         "--format",
         "-f",
         help="Output display format.",
@@ -106,8 +95,8 @@ def action_run(
         callback=principal_validator,
     ),
     verbose: bool = verbosity_option,
-    output_format: ActionOutputFormat = typer.Option(
-        ActionOutputFormat.json,
+    output_format: OutputFormat = typer.Option(
+        OutputFormat.json,
         "--format",
         "-f",
         help="Output display format.",
@@ -127,18 +116,32 @@ def action_run(
         "--label",
         "-l",
         help="Optional label to mark this execution of the action.",
-    )
+    ),
+    watch: bool = typer.Option(
+        False,
+        "--watch",
+        "-w",
+        help="Continuously poll this Action until it reaches a completed state.",
+        show_default=True,
+    ),
 ):
     """
     Launch an Action.
     """
-    ac = create_action_client(action_url, action_scope)
     parsed_body = process_input(body, input_format)
-    try:
-        result = ac.run(parsed_body, request_id, manage_by, monitor_by, label=label)
-    except GlobusAPIError as err:
-        result = err
-    format_and_echo(result, output_format.get_dumper(), verbose=verbose)
+    ac = create_action_client(action_url, action_scope)
+    method = functools.partial(
+        ac.run, parsed_body, request_id, manage_by, monitor_by, label=label
+    )
+
+    with live_content:
+        # Set watch to false here to immediately return after running the Action
+        result = request_runner(method, output_format, verbose, False)
+
+        if watch and not isinstance(result, GlobusAPIError):
+            action_id = result.data.get("action_id")
+            method = functools.partial(ac.status, action_id)
+            request_runner(method, output_format, verbose, watch)
 
 
 @app.command("status")
@@ -156,8 +159,55 @@ def action_status(
     ),
     action_id: str = typer.Argument(...),
     verbose: bool = verbosity_option,
-    output_format: ActionOutputFormat = typer.Option(
-        ActionOutputFormat.json,
+    output_format: OutputFormat = typer.Option(
+        OutputFormat.json,
+        "--format",
+        "-f",
+        help="Output display format.",
+        case_sensitive=False,
+        show_default=True,
+    ),
+    watch: bool = typer.Option(
+        False,
+        "--watch",
+        "-w",
+        help="Continuously poll this Action until it reaches a completed state. ",
+        show_default=True,
+    ),
+):
+    """
+    Query an Action's status by its ACTION_ID.
+    """
+    ac = create_action_client(action_url, action_scope)
+    method = functools.partial(ac.status, action_id)
+    with live_content:
+        request_runner(method, output_format, verbose, watch)
+
+
+@app.command("resume")
+def action_resume(
+    action_url: str = typer.Option(
+        ...,
+        help="The url at which the target Action Provider is located.",
+        prompt=True,
+        callback=url_validator_callback,
+    ),
+    action_scope: str = typer.Option(
+        None,
+        help="The scope this Action Provider uses to authenticate requests.",
+        callback=url_validator_callback,
+    ),
+    query_for_inactive_reason: bool = typer.Option(
+        True,
+        help=(
+            "Should the Action first be queried to determine the reason for the "
+            "resume, and prompt for additional consent if needed."
+        ),
+    ),
+    action_id: str = typer.Argument(...),
+    verbose: bool = verbosity_option,
+    output_format: OutputFormat = typer.Option(
+        OutputFormat.json,
         "--format",
         "-f",
         help="Output display format.",
@@ -166,11 +216,21 @@ def action_status(
     ),
 ):
     """
-    Query an Action's status by its ACTION_ID.
+    Resume an inactive Action by its ACTION_ID.
     """
-    ac = create_action_client(action_url, action_scope)
+    ac = create_action_client(action_url, action_scope=action_scope)
     try:
-        result = ac.status(action_id)
+        if query_for_inactive_reason:
+            result = ac.status(action_id)
+            body = result.data
+            status = body.get("status")
+            details = body.get("details", {})
+            code = details.get("code")
+            if status == "INACTIVE" and code == "ConsentRequired":
+                required_scope = details.get("required_scope")
+                if required_scope is not None:
+                    ac = create_action_client(action_url, action_scope=required_scope)
+        result = ac.resume(action_id)
     except GlobusAPIError as err:
         result = err
     format_and_echo(result, output_format.get_dumper(), verbose=verbose)
@@ -191,8 +251,8 @@ def action_cancel(
     ),
     action_id: str = typer.Argument(...),
     verbose: bool = verbosity_option,
-    output_format: ActionOutputFormat = typer.Option(
-        ActionOutputFormat.json,
+    output_format: OutputFormat = typer.Option(
+        OutputFormat.json,
         "--format",
         "-f",
         help="Output display format.",
@@ -226,8 +286,8 @@ def action_release(
     ),
     action_id: str = typer.Argument(...),
     verbose: bool = verbosity_option,
-    output_format: ActionOutputFormat = typer.Option(
-        ActionOutputFormat.json,
+    output_format: OutputFormat = typer.Option(
+        OutputFormat.json,
         "--format",
         "-f",
         help="Output display format.",
