@@ -1,11 +1,11 @@
+import functools
 import json
 from enum import Enum
-from typing import Any, List, Mapping
+from typing import Any, List, Mapping, Optional
 
 import typer
 import yaml
-from globus_sdk import GlobusAPIError, GlobusHTTPResponse
-
+from globus_automate_client.cli.auth import CLIENT_ID
 from globus_automate_client.cli.callbacks import (
     flow_input_validator,
     flows_endpoint_envvar_callback,
@@ -15,13 +15,21 @@ from globus_automate_client.cli.callbacks import (
     principal_validator,
     url_validator_callback,
 )
-from globus_automate_client.cli.constants import InputFormat
+from globus_automate_client.cli.constants import (
+    FlowDisplayFormat,
+    InputFormat,
+    OutputFormat,
+)
 from globus_automate_client.cli.helpers import (
-    display_http_details,
+    flow_log_runner,
     format_and_echo,
+    get_http_details,
+    parse_query_options,
     process_input,
+    request_runner,
     verbosity_option,
 )
+from globus_automate_client.cli.rich_rendering import live_content
 from globus_automate_client.client_helpers import create_flows_client
 from globus_automate_client.flows_client import (
     PROD_FLOWS_BASE_URL,
@@ -32,7 +40,7 @@ from globus_automate_client.graphviz_rendering import (
     graphviz_format,
     state_colors_for_log,
 )
-from globus_automate_client.token_management import CLIENT_ID
+from globus_sdk import GlobusAPIError, GlobusHTTPResponse
 
 
 class FlowRole(str, Enum):
@@ -42,13 +50,6 @@ class FlowRole(str, Enum):
     administered_by = "administered_by"
 
 
-class FlowDisplayFormat(str, Enum):
-    json = "json"
-    graphviz = "graphviz"
-    image = "image"
-    yaml = "yaml"
-
-
 class ActionRole(str, Enum):
     created_by = "created_by"
     monitor_by = "monitor_by"
@@ -193,7 +194,7 @@ def flow_deploy(
         )
     except GlobusAPIError as err:
         result = err
-    format_and_echo(result, input_format.get_dumper(), verbose=verbose)
+    format_and_echo(result, verbose=verbose)
 
 
 @app.command("update")
@@ -388,6 +389,25 @@ def flow_list(
         hidden=True,
         callback=flows_endpoint_envvar_callback,
     ),
+    filters: Optional[List[str]] = typer.Option(
+        None,
+        "--filter",
+        help="A filtering criteria in the form 'key=value' to apply to the "
+        "resulting Flow listing. The key indicates the filter, the value "
+        "indicates the pattern to match. Multiple patterns for a single key may "
+        "be specified as a comma seperated string, the results for which will "
+        "represent a logical OR. If multiple filters are applied, the returned "
+        "data will be the result of a logical AND between them. [repeatable]",
+    ),
+    orderings: Optional[List[str]] = typer.Option(
+        None,
+        "--orderby",
+        help="An ordering criteria in the form 'key=value' to apply to the resulting "
+        "Flow listing. The key indicates the field to order on, and the value is "
+        "either ASC, for ascending order, or DESC, for descending order. The first "
+        "ordering criteria will be used to sort the data, subsequent ordering criteria "
+        "will further sort ties. [repeatable]",
+    ),
     verbose: bool = verbosity_option,
     output_format: FlowDisplayFormat = typer.Option(
         FlowDisplayFormat.json,
@@ -401,10 +421,17 @@ def flow_list(
     """
     List Flows for which you have access.
     """
+    parsed_filters = parse_query_options(filters)
+    parsed_orderings = parse_query_options(orderings)
+
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
     try:
         flows = fc.list_flows(
-            roles=[r.value for r in roles], marker=marker, per_page=per_page
+            roles=[r.value for r in roles],
+            marker=marker,
+            per_page=per_page,
+            filters=parsed_filters,
+            orderings=parsed_orderings,
         )
     except GlobusAPIError as err:
         format_and_echo(err, verbose=verbose)
@@ -517,20 +544,34 @@ def flow_run(
         "--label",
         "-l",
         help="Optional label to mark this run.",
-    )
+    ),
+    watch: bool = typer.Option(
+        False,
+        "--watch",
+        "-w",
+        help="Continuously poll this Action until it reaches a completed state.",
+        show_default=True,
+    ),
 ):
     """
     Run an instance of a Flow. The argument provides the initial state of the Flow.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
     flow_input_dict = _process_flow_input(flow_input, input_format)
+    method = functools.partial(
+        fc.run_flow, flow_id, flow_scope, flow_input_dict, label=label
+    )
 
-    try:
-        response = fc.run_flow(flow_id, flow_scope, flow_input_dict, label=label)
-    except GlobusAPIError as err:
-        format_and_echo(err, verbose=verbose)
-    else:
-        _format_and_display_flow(response, output_format, verbose=verbose)
+    with live_content:
+        # Set watch to false here to immediately return after running the Action
+        response = request_runner(method, output_format, verbose, False)
+
+        if watch and isinstance(response, GlobusHTTPResponse):
+            action_id = response.data.get("action_id")
+            method = functools.partial(
+                fc.flow_action_status, flow_id, flow_scope, action_id
+            )
+            request_runner(method, output_format, verbose, watch)
 
 
 @app.command("action-list")
@@ -569,6 +610,25 @@ def flow_actions_list(
         min=1,
         max=50,
     ),
+    filters: Optional[List[str]] = typer.Option(
+        None,
+        "--filter",
+        help="A filtering criteria in the form 'key=value' to apply to the "
+        "resulting Action listing. The key indicates the filter, the value "
+        "indicates the pattern to match. Multiple patterns for a single key may "
+        "be specified as a comma seperated string, the results for which will "
+        "represent a logical OR. If multiple filters are applied, the returned "
+        "data will be the result of a logical AND between them. [repeatable]",
+    ),
+    orderings: Optional[List[str]] = typer.Option(
+        None,
+        "--orderby",
+        help="An ordering criteria in the form 'key=value' to apply to the resulting "
+        "Flow listing. The key indicates the field to order on, and the value is "
+        "either ASC, for ascending order, or DESC, for descending order. The first "
+        "ordering criteria will be used to sort the data, subsequent ordering criteria "
+        "will further sort ties. [repeatable]",
+    ),
     flows_endpoint: str = typer.Option(
         PROD_FLOWS_BASE_URL,
         hidden=True,
@@ -579,6 +639,8 @@ def flow_actions_list(
     """
     List a Flow definition's discrete invocations.
     """
+    parsed_filters = parse_query_options(filters)
+    parsed_orderings = parse_query_options(orderings)
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
 
     # This None check and check makes me unhappy but is necessary for mypy to
@@ -598,6 +660,8 @@ def flow_actions_list(
             roles=roles_str,
             marker=marker,
             per_page=per_page,
+            filters=parsed_filters,
+            orderings=parsed_orderings,
         )
     except GlobusAPIError as err:
         result = err
@@ -622,14 +686,69 @@ def flow_action_status(
         hidden=True,
         callback=flows_endpoint_envvar_callback,
     ),
+    watch: bool = typer.Option(
+        False,
+        "--watch",
+        "-w",
+        help="Continuously poll this Action until it reaches a completed state.",
+        show_default=True,
+    ),
     verbose: bool = verbosity_option,
 ):
     """
     Display the status for a Flow definition's particular invocation.
+    """
+    fc = create_flows_client(CLIENT_ID, flows_endpoint)
+    method = functools.partial(fc.flow_action_status, flow_id, flow_scope, action_id)
+
+    with live_content:
+        request_runner(method, OutputFormat.json, verbose, watch)
+
+
+@app.command("action-resume")
+def flow_action_resume(
+    action_id: str = typer.Argument(...),
+    flow_id: str = typer.Option(
+        ...,
+        help="The ID for the Flow which triggered the Action.",
+        prompt=True,
+    ),
+    flow_scope: str = typer.Option(
+        None,
+        help="The scope this Flow uses to authenticate requests.",
+        callback=url_validator_callback,
+    ),
+    query_for_inactive_reason: bool = typer.Option(
+        True,
+        help=(
+            "Should the Action first be queried to determine the reason for the "
+            "resume, and prompt for additional consent if needed."
+        ),
+    ),
+    flows_endpoint: str = typer.Option(
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
+    ),
+    verbose: bool = verbosity_option,
+):
+    """Resume a Flow in the INACTIVE state. If query-for-inactive-reason is set, and the
+    Flow Action is in an INACTIVE state due to requiring additional Consent, the required
+    Consent will be determined and you may be prompted to allow Consent using the Globus
+    Auth web interface.
+
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
     try:
-        result = fc.flow_action_status(flow_id, flow_scope, action_id)
+        if query_for_inactive_reason:
+            result = fc.flow_action_status(flow_id, flow_scope, action_id)
+            body = result.data
+            status = body.get("status")
+            details = body.get("details", {})
+            code = details.get("code")
+            if status == "INACTIVE" and code == "ConsentRequired":
+                flow_scope = details.get("required_scope")
+        result = fc.flow_action_resume(flow_id, flow_scope, action_id)
     except GlobusAPIError as err:
         result = err
     format_and_echo(result, verbose=verbose)
@@ -744,6 +863,15 @@ def flow_action_log(
         case_sensitive=False,
         show_default=True,
     ),
+    watch: bool = typer.Option(
+        False,
+        "--watch",
+        "-w",
+        help="Continuously poll this Action until it reaches a completed state. "
+        "Using this option will report only the latest state available."
+        "Only JSON and YAML output formats are supported.",
+        show_default=True,
+    ),
     flows_endpoint: str = typer.Option(
         PROD_FLOWS_BASE_URL,
         hidden=True,
@@ -755,30 +883,36 @@ def flow_action_log(
     Get a log of the steps executed by a Flow definition's invocation.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
+    method = functools.partial(
+        fc.flow_action_log,
+        flow_id,
+        flow_scope,
+        action_id,
+        limit,
+        reverse,
+        marker,
+        per_page,
+    )
+
+    if watch and output_format in {FlowDisplayFormat.json, FlowDisplayFormat.yaml}:
+        with live_content:
+            flow_log_runner(method, output_format, verbose, watch)
+        raise typer.Exit()
 
     try:
-        resp = fc.flow_action_log(
-            flow_id, flow_scope, action_id, limit, reverse, marker, per_page
-        )
+        resp = method()
     except GlobusAPIError as err:
-        format_and_echo(err, verbose=verbose)
-        raise typer.Exit(code=1)
+        resp = err
 
-    if verbose:
-        display_http_details(resp)
-
-    if output_format in (FlowDisplayFormat.json, FlowDisplayFormat.yaml):
-        _format_and_display_flow(resp, output_format, verbose)
-    elif output_format in (FlowDisplayFormat.graphviz, FlowDisplayFormat.image):
-        flow_def_resp = fc.get_flow(flow_id)
-        flow_def = flow_def_resp.data["definition"]
-        colors = state_colors_for_log(resp.data["entries"])
-        graphviz_out = graphviz_format(flow_def, colors)
-
-        if output_format == FlowDisplayFormat.graphviz:
-            typer.echo(graphviz_out.source)
+    if output_format in {FlowDisplayFormat.json, FlowDisplayFormat.yaml}:
+        format_and_echo(resp, output_format.get_dumper(), verbose=verbose)
+    else:
+        if isinstance(resp, GlobusHTTPResponse):
+            flow_def = fc.get_flow(flow_id)
+            dumper = output_format.get_dumper()
+            dumper(resp, flow_def)
         else:
-            graphviz_out.render("flows-output/graph", view=True, cleanup=True)
+            format_and_echo(resp, verbose=verbose)
 
 
 def _format_and_display_flow(
@@ -788,12 +922,12 @@ def _format_and_display_flow(
     Diplays a flow as either JSON, graphviz, or an image
     """
     if verbose:
-        display_http_details(flow_resp)
+        print(get_http_details(flow_resp))
 
     if output_format is FlowDisplayFormat.json:
-        format_and_echo(flow_resp, json.dumps)
+        format_and_echo(flow_resp, OutputFormat.json.get_dumper())
     elif output_format is FlowDisplayFormat.yaml:
-        format_and_echo(flow_resp, yaml.dump)
+        format_and_echo(flow_resp, OutputFormat.yaml.get_dumper())
     elif output_format in (FlowDisplayFormat.graphviz, FlowDisplayFormat.image):
         graphviz_out = graphviz_format(flow_resp.data["definition"])
         if output_format == FlowDisplayFormat.graphviz:
