@@ -94,6 +94,20 @@ def _all_vals_for_keys(
 
 
 def validate_flow_definition(flow_definition: Mapping[str, Any]) -> None:
+    """Perform local, JSONSchema based validation of a Flow definition. This is validation
+    on the basic structure of your Flow definition.such as required fields / properties
+    for the various state types and the overall structure of the Flow. This schema based
+    validation *does not* do any validation of input values or parameters passed to
+    Actions as those Actions define their own schemas and the Flow may generate or
+    compute values to these Actions and thus static, schema based validation cannot
+    determine if the Action parameter values generated during execution are correct.
+
+    The input is the dictionary containing the flow definition.
+
+    If the flow passes validation, no value is returned. If validation errors are found,
+    a FlowValidationError exception will be raised containing a string message describing
+    the error(s) encountered.
+    """
     schema_path = Path(__file__).parent / "flows_schema.json"
     with schema_path.open() as sf:
         flow_schema = json.load(sf)
@@ -321,6 +335,8 @@ class FlowsClient(BaseClient):
         roles: Optional[List[str]] = None,
         marker: Optional[str] = None,
         per_page: Optional[int] = None,
+        filters: Optional[dict] = None,
+        orderings: Optional[dict] = None,
         **kwargs,
     ) -> GlobusHTTPResponse:
         """
@@ -343,6 +359,18 @@ class FlowsClient(BaseClient):
             service and returned by operations that support pagination.
         :param per_page: The number of results to return per page. If
             supplied a pagination_token, this parameter has no effect.
+        :param filters: A filtering criteria to apply to the resulting Flow
+            listing. The keys indicate the filter, the values indicate the
+            pattern to match. The returned data will be the result of a logical
+            AND between the filters. Patterns may be comma separated to produce
+            the result of a logical OR.
+        :param orderings: An ordering criteria to apply to the resulting
+            Flow listing. The keys indicate the field to order on, and
+            the value can be either ASC, for ascending order, or DESC, for
+            descending order. The first ordering criteria will be used to sort
+            the data, subsequent ordering criteria will be applied for ties.
+            Note: To ensure orderings are applied in the correct order, use an
+            OrderedDict if trying to apply multiple orderings.
         """
         self.authorizer = self.flow_management_authorizer
         params = {}
@@ -352,6 +380,13 @@ class FlowsClient(BaseClient):
             params["pagination_token"] = marker
         if per_page is not None and marker is None:
             params["per_page"] = str(per_page)
+        if filters is not None:
+            params.update(filters)
+        if orderings is not None:
+            builder = []
+            for field, value in orderings.items():
+                builder.append(f"{field} {value}")
+            params["orderby"] = ",".join(builder)
         return self.get("/flows", params=params, **kwargs)
 
     def delete_flow(self, flow_id: str, **kwargs):
@@ -400,7 +435,7 @@ class FlowsClient(BaseClient):
 
         :param flow_scope:  The scope associated with the Flow ``flow_id``. If
             not provided, the SDK will attempt to perform an introspection on
-            the Flow to pull its scope automatically
+            the Flow to determine its scope automatically
 
         :param flow_input: A Flow-specific dictionary specifying the input
             required for the Flow to run.
@@ -425,7 +460,7 @@ class FlowsClient(BaseClient):
 
         :param flow_scope: The scope associated with the Flow ``flow_id``. If
             not provided, the SDK will attempt to perform an introspection on
-            the Flow to pull its scope automatically
+            the Flow to determine its scope automatically
 
         :param flow_action_id: The ID specifying the Action for which's status
             we want to query
@@ -440,6 +475,31 @@ class FlowsClient(BaseClient):
         ac = ActionClient.new_client(flow_url, authorizer)
         return ac.status(flow_action_id)
 
+    def flow_action_resume(
+        self, flow_id: str, flow_scope: Optional[str], flow_action_id: str, **kwargs
+    ) -> GlobusHTTPResponse:
+        """
+        Resume a Flow Action which is in an INACTIVE state.
+
+        :param flow_id: The UUID identifying the Flow which triggered the Action
+
+        :param flow_scope: The scope associated with the Flow ``flow_id``. If
+            not provided, the SDK will attempt to perform an introspection on
+            the Flow to determine its scope automatically.
+
+        :param flow_action_id: The ID specifying the Action with an INACTIVE
+            status we want to resume.
+
+        :param kwargs: Any additional kwargs passed into this method are passed
+            onto the Globus BaseClient. If there exists an "authorizer" keyword
+            argument, that gets used to run the Flow operation. Otherwise the
+            authorizer_callback defined for the FlowsClient will be used.
+        """
+        authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
+        flow_url = f"{self.base_url}/flows/{flow_id}"
+        ac = ActionClient.new_client(flow_url, authorizer)
+        return ac.resume(flow_action_id)
+
     def flow_action_release(
         self, flow_id: str, flow_scope: Optional[str], flow_action_id: str, **kwargs
     ) -> GlobusHTTPResponse:
@@ -450,7 +510,7 @@ class FlowsClient(BaseClient):
 
         :param flow_scope: The scope associated with the Flow ``flow_id``. If
             not provided, the SDK will attempt to perform an introspection on
-            the Flow to pull its scope automatically
+            the Flow to determine its scope automatically
 
         :param flow_action_id: The ID specifying the Action to release
 
@@ -474,7 +534,7 @@ class FlowsClient(BaseClient):
 
         :param flow_scope: The scope associated with the Flow ``flow_id``. If
             not provided, the SDK will attempt to perform an introspection on
-            the Flow to pull its scope automatically
+            the Flow to determine its scope automatically
 
         :param flow_action_id: The ID specifying the Action we want to cancel
 
@@ -496,6 +556,8 @@ class FlowsClient(BaseClient):
         roles: Optional[List[str]] = None,
         marker: Optional[str] = None,
         per_page: Optional[int] = None,
+        filters: Optional[dict] = None,
+        orderings: Optional[dict] = None,
         **kwargs,
     ) -> GlobusHTTPResponse:
         """
@@ -505,7 +567,7 @@ class FlowsClient(BaseClient):
 
         :param flow_scope: The scope associated with the Flow ``flow_id``. If
             not provided, the SDK will attempt to perform an introspection on
-            the Flow to pull its scope automatically
+            the Flow to determine its scope automatically
 
         :param statuses: A list of statuses used to filter the Actions that are
             returned by the listing. Returned Actions are guaranteed to have one
@@ -530,6 +592,18 @@ class FlowsClient(BaseClient):
             service and returned by operations that support pagination.
         :param per_page: The number of results to return per page. If
             supplied a pagination_token, this parameter has no effect.
+        :param filters: A filtering criteria to apply to the resulting Action
+            listing. The keys indicate the filter, the values indicate the
+            pattern to match. The returned data will be the result of a logical
+            AND between the filters. Patterns may be comma separated to produce
+            the result of a logical OR.
+        :param orderings: An ordering criteria to apply to the resulting
+            Action listing. The keys indicate the field to order on, and
+            the value can be either ASC, for ascending order, or DESC, for
+            descending order. The first ordering criteria will be used to sort
+            the data, subsequent ordering criteria will be applied for ties.
+            Note: To ensure orderings are applied in the correct order, use an
+            OrderedDict if trying to apply multiple orderings.
 
         :param kwargs: Any additional kwargs passed into this method are passed
             onto the Globus BaseClient. If there exists an "authorizer" keyword
@@ -540,13 +614,21 @@ class FlowsClient(BaseClient):
 
         params = {}
         if statuses is not None and len(statuses) > 0:
-            params.update(dict(status=",".join(statuses)))
+            params.update(dict(filter_status=",".join(statuses)))
         if roles is not None and len(roles) > 0:
-            params.update(dict(roles=",".join(roles)))
+            params.update(dict(filter_roles=",".join(roles)))
         if marker is not None:
             params["pagination_token"] = marker
         if per_page is not None and marker is None:
             params["per_page"] = str(per_page)
+        if filters:
+            params.update(filters)
+        if orderings:
+            builder = []
+            for field, value in orderings.items():
+                builder.append(f"{field} {value}")
+            params["orderby"] = ",".join(builder)
+
         return self.get(f"/flows/{flow_id}/actions", params=params, **kwargs)
 
     def flow_action_log(
@@ -568,7 +650,7 @@ class FlowsClient(BaseClient):
 
         :param flow_scope: The scope associated with the Flow ``flow_id``. If
             not provided, the SDK will attempt to perform an introspection on
-            the Flow to pull its scope automatically
+            the Flow to determine its scope automatically
 
         :param flow_action_id: The ID specifying the Action for which's history
             to query
@@ -602,6 +684,7 @@ class FlowsClient(BaseClient):
         authorizer_callback: AuthorizerCallbackType,
         authorizer: AllowedAuthorizersType,
         base_url: str = PROD_FLOWS_BASE_URL,
+        http_timeout: int = 10,
     ) -> _FlowsClient:
         """
         Classmethod to simplify creating an FlowsClient. Use this method when
@@ -647,4 +730,5 @@ class FlowsClient(BaseClient):
             app_name="Globus Automate SDK FlowsClient",
             base_url=base_url,
             authorizer=authorizer,
+            http_timeout=http_timeout,
         )
